package com.rf.psychological.scale.resultBusiness.scaleResult;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.rf.psychological.utils.Constant;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @description 情绪一社交孤独问卷
 */
public class ESLISScale extends BaseScale {
    public ESLISScale(JSONArray jsonArray, JSONObject resultJson) {
        super(jsonArray, resultJson);
    }

    public JSONObject scaleCalculate() throws Exception {
        //生活中的状况总分
        int score1 = 0;
        //生活中的体验总分
        int score2 = 0;
        //情绪孤立与孤独总分
        int score3 = 0;
        //社交孤立与孤独总分
        int score4 = 0;

        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObject1 = jsonArray.getJSONObject(i);
            if ((i + 1) % 2 == 0) {
                if (jsonObject1.getString("checkItems").equals("偶尔如此")) {
                    score2 += 0;
                } else if (jsonObject1.getString("checkItems").equals("有时如此")) {
                    score2 += 1;
                } else if (jsonObject1.getString("checkItems").equals("经常如此")) {
                    score2 += 2;
                } else if (jsonObject1.getString("checkItems").equals("通常如此")) {
                    score2 += 3;
                }
            } else {
                if (jsonObject1.getString("checkItems").equals("偶尔如此")) {
                    score1 += 0;
                } else if (jsonObject1.getString("checkItems").equals("有时如此")) {
                    score1 += 1;
                } else if (jsonObject1.getString("checkItems").equals("经常如此")) {
                    score1 += 2;
                } else if (jsonObject1.getString("checkItems").equals("通常如此")) {
                    score1 += 3;
                }
            }

            if ((i + 1) <= 16) {
                if (jsonObject1.getString("checkItems").equals("偶尔如此")) {
                    score3 += 0;
                } else if (jsonObject1.getString("checkItems").equals("有时如此")) {
                    score3 += 1;
                } else if (jsonObject1.getString("checkItems").equals("经常如此")) {
                    score3 += 2;
                } else if (jsonObject1.getString("checkItems").equals("通常如此")) {
                    score3 += 3;
                }
            } else {
                if (jsonObject1.getString("checkItems").equals("偶尔如此")) {
                    score4 += 0;
                } else if (jsonObject1.getString("checkItems").equals("有时如此")) {
                    score4 += 1;
                } else if (jsonObject1.getString("checkItems").equals("经常如此")) {
                    score4 += 2;
                } else if (jsonObject1.getString("checkItems").equals("通常如此")) {
                    score4 += 3;
                }
            }

        }


        //得分列表
        Map<String, String> resultMap = new LinkedHashMap<>();
        resultMap.put("生活中的状况总分", String.valueOf(score1));
        resultMap.put("生活中的体验总分", String.valueOf(score2));
        resultMap.put("情绪孤立与孤独总分", String.valueOf(score3));
        resultMap.put("社交孤立与孤独总分", String.valueOf(score4));

        resultJson.put("生活中的状况总分", String.valueOf(score1));
        resultJson.put("生活中的体验总分", String.valueOf(score2));
        resultJson.put("情绪孤立与孤独总分", String.valueOf(score3));
        resultJson.put("社交孤立与孤独总分", String.valueOf(score4));


        if (score1 < 6) {
            resultMap.put("对孤立的结论", "表示无或几无孤立");
            resultJson.put("对孤立的结论", "表示无或几无孤立");
        } else if (score1 <= 8) {
            resultMap.put("对孤立的结论", "表示一般的孤立");
            resultJson.put("对孤立的结论", "表示一般的孤立");
        } else if (score1 <= 12) {
            resultMap.put("对孤立的结论", "表示高于一般人的孤立");
            resultJson.put("对孤立的结论", "表示高于一般人的孤立");
        } else {
            resultMap.put("对孤立的结论", "表示孤立问题严重。");
            resultJson.put("对孤立的结论", "表示孤立问题严重。");
        }

        if (score3 < 6) {
            resultMap.put("对情绪孤独的结论", "表示无或几无孤独");
            resultJson.put("对情绪孤独的结论", "表示无或几无孤独");
        } else if (score3 <= 10) {
            resultMap.put("对情绪孤独的结论", "表示一般的孤独");
            resultJson.put("对情绪孤独的结论", "表示一般的孤独");
        } else if (score3 <= 14) {
            resultMap.put("对情绪孤独的结论", "表示高于一般人的孤独");
            resultJson.put("对情绪孤独的结论", "表示高于一般人的孤独");
        } else {
            resultMap.put("对情绪孤独的结论", "表示情绪孤独问题严重。");
            resultJson.put("对情绪孤独的结论", "表示情绪孤独问题严重。");
        }


        if (score4 <= 4) {
            resultMap.put("对社交孤独的结论", "表示无或几无孤独");
            resultJson.put("对社交孤独的结论", "表示无或几无孤独");
        } else if (score4 <= 9) {
            resultMap.put("对社交孤独的结论", "表示一般的孤独");
            resultJson.put("对社交孤独的结论", "表示一般的孤独");
        } else if (score4 <= 13) {
            resultMap.put("对社交孤独的结论", "表示高于一般人的孤独");
            resultJson.put("对社交孤独的结论", "表示高于一般人的孤独");
        } else {
            resultMap.put("对社交孤独的结论", "表示社交孤独问题严重。");
            resultJson.put("对社交孤独的结论", "表示社交孤独问题严重。");
        }



        List<Map<String, String>> resultMapList = new ArrayList<>();
        resultMapList.add(resultMap);

        JSONObject returnJson = new JSONObject(true);
        returnJson.put("resultMapList", resultMapList);
        returnJson.put("scaleName", Constant.SHEET_NAME_ESLIS);
        returnJson.put("resultJson", resultJson);

        return returnJson;
    }
}
